package sun.util.locale;

import java.lang.ref.SoftReference;

public final class BaseLocale
{
    public static final String SEP = "_";
    private static final BaseLocale.Cache CACHE;
    private final String language;
    private final String script;
    private final String region;
    private final String variant;
    private volatile int hash;
    
    private BaseLocale(String language, String region) {
        this.hash = 0;
        this.language = language;
        this.script = "";
        this.region = region;
        this.variant = "";
    }
    
    private BaseLocale(String s, String s2, String s3, String s4) {
        this.hash = 0;
        this.language = ((s != null) ? LocaleUtils.toLowerString(s).intern() : "");
        this.script = ((s2 != null) ? LocaleUtils.toTitleString(s2).intern() : "");
        this.region = ((s3 != null) ? LocaleUtils.toUpperString(s3).intern() : "");
        this.variant = ((s4 != null) ? s4.intern() : "");
    }
    
    public static BaseLocale createInstance(String s, String s2) {
        BaseLocale baseLocale = new BaseLocale(s, s2);
        BaseLocale.CACHE.put(new Key(s, s2), baseLocale);
        return baseLocale;
    }
    
    public static BaseLocale getInstance(String s, String s2, String s3, String s4) {
        if (s != null) {
            if (LocaleUtils.caseIgnoreMatch(s, "he")) {
                s = "iw";
            }
            else if (LocaleUtils.caseIgnoreMatch(s, "yi")) {
                s = "ji";
            }
            else if (LocaleUtils.caseIgnoreMatch(s, "id")) {
                s = "in";
            }
        }
        return (BaseLocale)BaseLocale.CACHE.get(new Key(s, s2, s3, s4));
    }
    
    public String getLanguage() {
        return this.language;
    }
    
    public String getScript() {
        return this.script;
    }
    
    public String getRegion() {
        return this.region;
    }
    
    public String getVariant() {
        return this.variant;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof BaseLocale)) {
            return false;
        }
        BaseLocale baseLocale = (BaseLocale)o;
        return this.language == baseLocale.language && this.script == baseLocale.script && this.region == baseLocale.region && this.variant == baseLocale.variant;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        if (this.language.length() > 0) {
            sb.append("language=");
            sb.append(this.language);
        }
        if (this.script.length() > 0) {
            if (sb.length() > 0) {
                sb.append(", ");
            }
            sb.append("script=");
            sb.append(this.script);
        }
        if (this.region.length() > 0) {
            if (sb.length() > 0) {
                sb.append(", ");
            }
            sb.append("region=");
            sb.append(this.region);
        }
        if (this.variant.length() > 0) {
            if (sb.length() > 0) {
                sb.append(", ");
            }
            sb.append("variant=");
            sb.append(this.variant);
        }
        return sb.toString();
    }
    
    @Override
    public int hashCode() {
        int hash = this.hash;
        if (hash == 0) {
            hash = 31 * (31 * (31 * this.language.hashCode() + this.script.hashCode()) + this.region.hashCode()) + this.variant.hashCode();
            this.hash = hash;
        }
        return hash;
    }
    
    BaseLocale(String s, String s2, String s3, String s4, BaseLocale.1 object) {
        this(s, s2, s3, s4);
    }
    
    static {
        CACHE = new BaseLocale.Cache();
    }
    
    private static final class Key
    {
        private final SoftReference<String> lang;
        private final SoftReference<String> scrt;
        private final SoftReference<String> regn;
        private final SoftReference<String> vart;
        private final boolean normalized;
        private final int hash;
        
        private Key(String referent, String referent2) {
            assert referent.intern() == referent && referent2.intern() == referent2;
            this.lang = new SoftReference(referent);
            this.scrt = new SoftReference("");
            this.regn = new SoftReference(referent2);
            this.vart = new SoftReference("");
            this.normalized = true;
            int hashCode = referent.hashCode();
            if (referent2 != "") {
                for (int length = referent2.length(), i = 0; i < length; ++i) {
                    hashCode = 31 * hashCode + LocaleUtils.toLower(referent2.charAt(i));
                }
            }
            this.hash = hashCode;
            return;
        }
        
        public Key(String s, String s2, String s3, String s4) {
            this(s, s2, s3, s4, false);
        }
        
        private Key(String referent, String referent2, String referent3, String referent4, boolean normalized) {
            int hash = 0;
            if (referent != null) {
                this.lang = new SoftReference(referent);
                for (int length = referent.length(), i = 0; i < length; ++i) {
                    hash = 31 * hash + LocaleUtils.toLower(referent.charAt(i));
                }
            }
            else {
                this.lang = new SoftReference("");
            }
            if (referent2 != null) {
                this.scrt = new SoftReference(referent2);
                for (int length2 = referent2.length(), j = 0; j < length2; ++j) {
                    hash = 31 * hash + LocaleUtils.toLower(referent2.charAt(j));
                }
            }
            else {
                this.scrt = new SoftReference("");
            }
            if (referent3 != null) {
                this.regn = new SoftReference(referent3);
                for (int length3 = referent3.length(), k = 0; k < length3; ++k) {
                    hash = 31 * hash + LocaleUtils.toLower(referent3.charAt(k));
                }
            }
            else {
                this.regn = new SoftReference("");
            }
            if (referent4 != null) {
                this.vart = new SoftReference(referent4);
                for (int length4 = referent4.length(), l = 0; l < length4; ++l) {
                    hash = 31 * hash + referent4.charAt(l);
                }
            }
            else {
                this.vart = new SoftReference("");
            }
            this.hash = hash;
            this.normalized = normalized;
        }
        
        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (o instanceof BaseLocale.Key && this.hash == ((BaseLocale.Key)o).hash) {
                String s = (String)this.lang.get();
                String s2 = (String)((BaseLocale.Key)o).lang.get();
                if (s != null && s2 != null && LocaleUtils.caseIgnoreMatch(s2, s)) {
                    String s3 = (String)this.scrt.get();
                    String s4 = (String)((BaseLocale.Key)o).scrt.get();
                    if (s3 != null && s4 != null && LocaleUtils.caseIgnoreMatch(s4, s3)) {
                        String s5 = (String)this.regn.get();
                        String s6 = (String)((BaseLocale.Key)o).regn.get();
                        if (s5 != null && s6 != null && LocaleUtils.caseIgnoreMatch(s6, s5)) {
                            String anObject = (String)this.vart.get();
                            String s7 = (String)((BaseLocale.Key)o).vart.get();
                            return s7 != null && s7.equals(anObject);
                        }
                    }
                }
            }
            return false;
        }
        
        @Override
        public int hashCode() {
            return this.hash;
        }
        
        public static BaseLocale.Key normalize(BaseLocale.Key key) {
            if (key.normalized) {
                return key;
            }
            return new BaseLocale.Key(LocaleUtils.toLowerString((String)key.lang.get()).intern(), LocaleUtils.toTitleString((String)key.scrt.get()).intern(), LocaleUtils.toUpperString((String)key.regn.get()).intern(), ((String)key.vart.get()).intern(), true);
        }
    }
    
    private static class Cache extends LocaleObjectCache<BaseLocale.Key, BaseLocale>
    {
        public Cache() {
        }
        
        @Override
        protected BaseLocale.Key normalizeKey(BaseLocale.Key key) {
            assert key.lang.get() != null && key.scrt.get() != null && key.regn.get() != null && key.vart.get() != null;
            return BaseLocale.Key.normalize(key);
        }
        
        @Override
        protected BaseLocale createObject(BaseLocale.Key key) {
            return new BaseLocale((String)key.lang.get(), (String)key.scrt.get(), (String)key.regn.get(), (String)key.vart.get());
        }
    }
}
